// Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License"). You may not
// use this file except in compliance with the License. A copy of the
// License is located at
//
// http://aws.amazon.com/apache2.0/
//
// or in the "license" file accompanying this file. This file is distributed
// on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
// either express or implied. See the License for the specific language governing
// permissions and limitations under the License.
//
// +build windows

// utility package implements all the shared methods between clients.
package utility

import (
	"math/rand"
	"time"
)

const (
	digits                = "0123456789"
	uppercaseLetters      = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
	lowercaseLetters      = "abcdefghijklmnopqrstuvwxyz"
	all                   = digits + uppercaseLetters + lowercaseLetters
	defaultPasswordLength = 10
)

type ISessionUtil interface {
	GeneratePasswordForDefaultUser() string
}

type SessionUtil struct {
	PasswordLength int
}

// GeneratePasswordForDefaultUser generates password for default ssm-user.
func (u *SessionUtil) GeneratePasswordForDefaultUser() string {
	if u.PasswordLength <= 0 {
		u.PasswordLength = defaultPasswordLength
	}

	digitsLen := len(digits)
	uppercaseLettersLen := len(uppercaseLetters)
	lowercaseLettersLen := len(lowercaseLetters)
	allLen := len(all)

	rand.Seed(time.Now().UnixNano())
	buf := make([]byte, u.PasswordLength)

	// atleast one digit
	buf[0] = digits[rand.Intn(digitsLen)]

	// atleast one uppercase letter
	buf[1] = uppercaseLetters[rand.Intn(uppercaseLettersLen)]

	// atleast one lowercase letter
	buf[2] = lowercaseLetters[rand.Intn(lowercaseLettersLen)]

	// mix up the rest
	for i := 3; i < u.PasswordLength; i++ {
		buf[i] = all[rand.Intn(allLen)]
	}

	return string(buf)
}
